@@ -430,6 +430,7 @@ _public_ int sd_bus_creds_get_user_unit(sd_bus_creds *c, const char **ret) {
 #endif // 0
 
 _public_ int sd_bus_creds_get_slice(sd_bus_creds *c, const char **ret) {
+#if 0 /// The "Slice" is the session in elogind
         int r;
 
         assert_return(c, -EINVAL);
@@ -454,9 +455,13 @@ _public_ int sd_bus_creds_get_slice(sd_bus_creds *c, const char **ret) {
 
         *ret = c->slice;
         return 0;
+#else
+        return sd_bus_creds_get_session(c, ret);
+#endif // 0
 }
 
 _public_ int sd_bus_creds_get_user_slice(sd_bus_creds *c, const char **ret) {
+#if 0 /// The "User Slice" is the session in elogind, too.
         int r;
 
         assert_return(c, -EINVAL);
@@ -481,6 +486,9 @@ _public_ int sd_bus_creds_get_user_slice(sd_bus_creds *c, const char **ret) {
 
         *ret = c->user_slice;
         return 0;
+#else
+        return sd_bus_creds_get_session(c, ret);
+#endif // 0
 }
 
 _public_ int sd_bus_creds_get_session(sd_bus_creds *c, const char **ret) {
