@@ -59,8 +59,14 @@ int bus_message_dump(sd_bus_message *m, FILE *f, unsigned flags) {
                         "%s%s%s Type=%s%s%s  Endian=%c  Flags=%u  Version=%u  Priority=%"PRIi64,
                         m->header->type == SD_BUS_MESSAGE_METHOD_ERROR ? ansi_highlight_red() :
                         m->header->type == SD_BUS_MESSAGE_METHOD_RETURN ? ansi_highlight_green() :
-                        m->header->type != SD_BUS_MESSAGE_SIGNAL ? ansi_highlight() : "", special_glyph(TRIANGULAR_BULLET), ansi_normal(),
-                        ansi_highlight(), bus_message_type_to_string(m->header->type), ansi_normal(),
+                        m->header->type != SD_BUS_MESSAGE_SIGNAL ? ansi_highlight() : "",
+                        special_glyph(TRIANGULAR_BULLET),
+                        ansi_normal(),
+
+                        ansi_highlight(),
+                        bus_message_type_to_string(m->header->type) ?: "(unknown)",
+                        ansi_normal(),
+
                         m->header->endian,
                         m->header->flags,
                         m->header->version,
@@ -321,8 +327,11 @@ int bus_creds_dump(sd_bus_creds *c, FILE *f, bool terse) {
         uint32_t audit_sessionid;
         char **cmdline = NULL, **well_known = NULL;
         const char *prefix, *color, *suffix, *s;
+#if 0 /// elogind does not support systemd units and slices. v and w are only used with them
         int r, q, v, w, z;
-
+#else
+        int r, q, z;
+#endif // 0
         assert(c);
 
         if (!f)
@@ -429,34 +438,32 @@ int bus_creds_dump(sd_bus_creds *c, FILE *f, bool terse) {
         if (c->mask & SD_BUS_CREDS_CGROUP)
                 fprintf(f, "%sCGroup=%s%s%s", prefix, color, c->cgroup, suffix);
         s = NULL;
-#if 0 /// elogind does not support systemd units
+#if 0 /// elogind does not support systemd units and slices (Would only display the session anyway)
         r = sd_bus_creds_get_unit(c, &s);
         if (r != -ENODATA)
                 fprintf(f, "%sUnit=%s%s%s", prefix, color, strna(s), suffix);
         s = NULL;
-#endif // 0
         v = sd_bus_creds_get_slice(c, &s);
         if (v != -ENODATA)
                 fprintf(f, "%sSlice=%s%s%s", prefix, color, strna(s), suffix);
         s = NULL;
-#if 0 /// elogind does not support systemd units
         q = sd_bus_creds_get_user_unit(c, &s);
         if (q != -ENODATA)
                 fprintf(f, "%sUserUnit=%s%s%s", prefix, color, strna(s), suffix);
         s = NULL;
-#endif // 0
         w = sd_bus_creds_get_user_slice(c, &s);
         if (w != -ENODATA)
                 fprintf(f, "%sUserSlice=%s%s%s", prefix, color, strna(s), suffix);
         s = NULL;
+#endif // 0
         z = sd_bus_creds_get_session(c, &s);
         if (z != -ENODATA)
                 fprintf(f, "%sSession=%s%s%s", prefix, color, strna(s), suffix);
 
-#if 0 /// elogind does not support systemd units, and q is only used with them until now
+#if 0 /// elogind does not support systemd units and slices. q, v and w are only used with them
         if (terse && ((c->mask & SD_BUS_CREDS_CGROUP) || r != -ENODATA || q != -ENODATA || v != -ENODATA || w != -ENODATA || z != -ENODATA))
 #else
-        if (terse && ((c->mask & SD_BUS_CREDS_CGROUP) || r != -ENODATA || v != -ENODATA || w != -ENODATA || z != -ENODATA))
+        if (terse && ((c->mask & SD_BUS_CREDS_CGROUP) || r != -ENODATA || z != -ENODATA))
 #endif // 0
                 fputs("\n", f);
 
