@@ -1,20 +1,5 @@
+/* SPDX-License-Identifier: LGPL-2.1+ */
 /***
-  This file is part of systemd.
-
-  Copyright 2011 Lennart Poettering
-
-  systemd is free software; you can redistribute it and/or modify it
-  under the terms of the GNU Lesser General Public License as published by
-  the Free Software Foundation; either version 2.1 of the License, or
-  (at your option) any later version.
-
-  systemd is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public License
-  along with systemd; If not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #include <errno.h>
@@ -30,6 +15,7 @@
 #include "io-util.h"
 #include "khash.h"
 #include "macro.h"
+#include "missing.h"
 #include "random-util.h"
 #include "user-util.h"
 #include "util.h"
@@ -65,7 +51,7 @@ _public_ int sd_id128_from_string(const char s[], sd_id128_t *ret) {
 
                         if (i == 8)
                                 is_guid = true;
-                        else if (i == 13 || i == 18 || i == 23) {
+                        else if (IN_SET(i, 13, 18, 23)) {
                                 if (!is_guid)
                                         return -EINVAL;
                         } else
@@ -105,11 +91,15 @@ _public_ int sd_id128_get_machine(sd_id128_t *ret) {
 
         if (sd_id128_is_null(saved_machine_id)) {
                 r = id128_read("/etc/machine-id", ID128_PLAIN, &saved_machine_id);
+#if 1 /// elogind supports setups, where the machine-id is in the dbus default path
+                if (r < 0)
+                        r = id128_read("/var/lib/dbus/machine-id", ID128_PLAIN, &saved_machine_id);
+#endif // 1
                 if (r < 0)
                         return r;
 
                 if (sd_id128_is_null(saved_machine_id))
-                        return -EINVAL;
+                        return -ENOMEDIUM;
         }
 
         *ret = saved_machine_id;
