@@ -26,7 +26,7 @@
 #include "special.h"
 #include "stdio-util.h"
 #include "string-table.h"
-//#include "strv.h"
+#include "strv.h"
 #include "unit-name.h"
 #include "user-util.h"
 //#include "util.h"
@@ -112,6 +112,8 @@ User *user_free(User *u) {
         if (!u)
                 return NULL;
 
+        log_debug_elogind("Freeing User %s ...", u->name);
+
         if (u->in_gc_queue)
                 LIST_REMOVE(gc_queue, u->manager->user_gc_queue, u);
 
@@ -341,6 +343,7 @@ int user_load(User *u) {
                         log_debug_errno(r, "Failed to parse 'STOPPING' boolean: %s", stopping);
                 else
                         u->stopping = r;
+                log_debug_elogind(" --> User stopping    : %d", u->stopping);
         }
 
         if (realtime)
@@ -350,6 +353,30 @@ int user_load(User *u) {
         if (last_session_timestamp)
                 (void) timestamp_deserialize(last_session_timestamp, &u->last_session_timestamp);
 
+        log_debug_elogind(" --> User realtime    : %lu", u->timestamp.realtime);
+        log_debug_elogind(" --> User monotonic   : %lu", u->timestamp.monotonic);
+        log_debug_elogind(" --> User last session: %lu", u->last_session_timestamp);
+#if ENABLE_DEBUG_ELOGIND
+        _cleanup_strv_free_ char **sa = NULL;
+        _cleanup_free_ char *sv = NULL;
+        Session *i;
+        bool first;
+
+        first = true;
+        LIST_FOREACH(sessions_by_user, i, u->sessions) {
+                if (first)
+                        first = false;
+                else
+                        strv_extend(&sa, " ");
+                strv_extend(&sa, i->id);
+        }
+
+        if (false == first) {
+                sv = strv_join(sa, " ");
+        }
+#endif // ENABLE_DEBUG_ELOGIND
+        log_debug_elogind(" --> User sessions    : %s",  u->sessions ? sv : "none");
+
         return 0;
 }
 
@@ -393,7 +420,6 @@ int user_start(User *u) {
         if (r < 0)
                 return r;
 #endif // 1
-
         /* Save the user data so far, because pam_systemd will read the XDG_RUNTIME_DIR out of it while starting up
          * systemd --user.  We need to do user_save_internal() because we have not "officially" started yet. */
         user_save_internal(u);
@@ -453,6 +479,8 @@ int user_stop(User *u, bool force) {
                 return 0;
         }
 
+        log_debug_elogind("Stopping user %s %s ...", u->name, force ? "(forced)" : "");
+
         LIST_FOREACH(sessions_by_user, s, u->sessions) {
                 int k;
 
@@ -461,8 +489,10 @@ int user_stop(User *u, bool force) {
                         r = k;
         }
 
-#if 0 /// elogind does not support service or slice jobs
+#if 0 /// elogind does not support service or slice jobs...
         user_stop_service(u);
+#else
+        user_add_to_gc_queue(u);
 #endif // 0
 
         u->stopping = true;
@@ -483,6 +513,8 @@ int user_finalize(User *u) {
 
         if (u->started)
                 log_debug("User %s logged out.", u->name);
+        else
+                log_debug_elogind("User %s not started, finalizing...", u->name);
 
         LIST_FOREACH(sessions_by_user, s, u->sessions) {
                 k = session_finalize(s);
@@ -604,6 +636,12 @@ bool user_may_gc(User *u, bool drop_not_started) {
 
         assert(u);
 
+        log_debug_elogind("User %s may gc ?", u->name);
+        log_debug_elogind("  dns && !started: %s", yes_no(drop_not_started && !u->started));
+        log_debug_elogind("  is sessionless : %s", yes_no(!u->sessions));
+        log_debug_elogind("  not lingering  : %s", yes_no(user_check_linger_file(u) > 0));
+        log_debug_elogind("  dns or stopping: %s", yes_no(drop_not_started || u->stopping));
+
         if (drop_not_started && !u->started)
                 return true;
 
@@ -641,11 +679,15 @@ bool user_may_gc(User *u, bool drop_not_started) {
                 if (r != 0)
                         return false;
         }
+
         /* Note that we don't care if the three units we manage for each user object are up or not, as we are managing
          * their state rather than tracking it. */
-#endif // 0
 
         return true;
+#else
+        // elogind has to rely on drop_not_started and that the state is correctly loaded
+        return (drop_not_started || u->stopping);
+#endif // 0
 }
 
 void user_add_to_gc_queue(User *u) {
