@@ -174,21 +174,21 @@ int sd_is_mq(int fd, const char *path);
   newline separated environment-style variable assignments in a
   string. The following variables are known:
 
-     MAINPID=...  The main PID of a daemon, in case elogind did not
+     MAINPID=...  The main PID of a daemon, in case systemd did not
                   fork off the process itself. Example: "MAINPID=4711"
 
-     READY=1      Tells elogind that daemon startup or daemon reload
+     READY=1      Tells systemd that daemon startup or daemon reload
                   is finished (only relevant for services of Type=notify).
                   The passed argument is a boolean "1" or "0". Since there
                   is little value in signaling non-readiness the only
                   value daemons should send is "READY=1".
 
-     RELOADING=1  Tell elogind that the daemon began reloading its
+     RELOADING=1  Tell systemd that the daemon began reloading its
                   configuration. When the configuration has been
                   reloaded completely, READY=1 should be sent to inform
-                  elogind about this.
+                  systemd about this.
 
-     STOPPING=1   Tells elogind that the daemon is about to go down.
+     STOPPING=1   Tells systemd that the daemon is about to go down.
 
      STATUS=...   Passes a single-line status string back to systemd
                   that describes the daemon state. This is free-form
